// Example for the MI CCD driver library.
//
// This example is distributed under the New BSD license:
//
// Copyright 2016, Jakub Smutny <jakub@mii.cz>
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
//
// 3. Neither the name of the copyright holder nor the names of its contributors
// may be used to endorse or promote products derived from this software without
// specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

#ifdef __APPLE__
# include <stdio.h>
#else
# include <malloc.h>
#endif
#include <stdlib.h>
#include <unistd.h>

#include "gxccd.h"

static int id = -1;

// This example uses only one camera. In case of multiple cameras connected,
// last one enumerated is used.
void callback(int eid)
{
  id = eid;
}

// This example does not check return values!
int main (void)
{
  int rv;
  float valf = 0.0;
  char buf[256] = {0};

  camera_t *camera = NULL;

  // configure your ip and port of ETH adapter here
  //gxccd_configure_eth("192.168.1.100", 0);
  // call gxccd_enumerate_eth for ETH Adapter variant
  //gxccd_enumerate_eth(callback);

  // call gxccd_enumerate_usb for USB variant
  gxccd_enumerate_usb(callback);
  printf("Using camera id:\t%d\n", id);

  // call gxccd_initialize_eth for ETH Adapter variant
  //camera = gxccd_initialize_eth(id);

  // call gxccd_initialize_usb for USB variant
  camera = gxccd_initialize_usb(id);
  if (!camera)
    return -1;

  gxccd_get_string_parameter(camera, GSP_CAMERA_DESCRIPTION, buf, sizeof(buf));
  printf("Camera description:\t%s\n", buf);

  int major, minor, build;
  gxccd_get_integer_parameter(camera, GIP_FIRMWARE_MAJOR, &major);
  gxccd_get_integer_parameter(camera, GIP_FIRMWARE_MINOR, &minor);
  gxccd_get_integer_parameter(camera, GIP_FIRMWARE_BUILD, &build);
  printf("Camera FW version:\t%d.%d.%d\n", major, minor, build);

  gxccd_get_value(camera, GV_CHIP_TEMPERATURE, &valf);
  printf("Camera chip temp:\t%.2f °C\n", valf);

  gxccd_get_value(camera, GV_SUPPLY_VOLTAGE, &valf);
  printf("Camera supply voltage:\t%.2f V\n", valf);

  int i = 0;
  do {
    rv = gxccd_enumerate_read_modes(camera, i, buf, sizeof(buf));
    if (!rv) {
      printf("Read mode #%d:\t\t%s\n", i, buf);
      i++;
    }
  } while (!rv);

  // set last read mode
  gxccd_set_read_mode(camera, i - 1);

  // start exposure
  const double exp_time = 0.3;
  int chip_w, chip_d;

  gxccd_get_integer_parameter(camera, GIP_CHIP_W, &chip_w);
  gxccd_get_integer_parameter(camera, GIP_CHIP_D, &chip_d);
  printf("Starting exposure of light frame with params:\nexp. time:\t\t%.2f"
         " sec\nwidth:\t\t\t%d px\nheight:\t\t\t%d px\n", exp_time, chip_w,
         chip_d);

  gxccd_start_exposure(camera, exp_time, true, 0, 0, chip_w, chip_d);
  // sleep during exposure
  sleep(exp_time + 1.0);

  bool ready = false;
  while (!ready) {
    gxccd_image_ready(camera, &ready);
  }

  // read whole frame
  size_t image_size = chip_w * chip_d * 2;
  char *image = malloc(image_size);
  if (!image)
    return -1;

  rv = gxccd_read_image(camera, image, image_size);
  if (rv)
    printf("Error reading image\n");
  else
    printf("Downloaded image size:\t%zu bytes\n", image_size);

  free(image);

  // disconnect
  gxccd_release(camera);

  return 0;
}
